diff --git a/encoder/analyse.c b/encoder/analyse.c
index af3185f..ac6e4ad 100644
--- a/encoder/analyse.c
+++ b/encoder/analyse.c
@@ -2376,7 +2376,10 @@ void x264_macroblock_analyse( x264_t *h )
                          if( h->mb.i_sub_partition[i8x8] == D_L0_8x8 )
                              x264_me_refine_qpel_rd( h, &analysis.l0.me8x8[i8x8], analysis.i_lambda2, i8x8 );
                          else
+                         {
                              x264_mb_cache_mv_p8x8( h, &analysis, i8x8, 1 );
+                             x264_macroblock_encode_p8x8( h, i8x8 );
+                         }
                 }
             }
         }
diff --git a/encoder/macroblock.c b/encoder/macroblock.c
index 67bc2d8..6038b1a 100644
--- a/encoder/macroblock.c
+++ b/encoder/macroblock.c
@@ -763,8 +763,9 @@ void x264_macroblock_encode_p8x8( x264_t *h, int i8 )
     uint8_t *p_fenc = h->mb.pic.p_fenc[0] + (i8&1)*8 + (i8>>1)*8*FENC_STRIDE;
     uint8_t *p_fdec = h->mb.pic.p_fdec[0] + (i8&1)*8 + (i8>>1)*8*FDEC_STRIDE;
     int b_decimate = h->sh.i_type == SLICE_TYPE_B || h->param.analyse.b_dct_decimate;
-    int nnz8x8;
+    int nnz8x8, nnz4x4;
     int ch;
+    int i4;
 
     x264_mb_mc_8x8( h, i8 );
 
@@ -780,6 +781,9 @@ void x264_macroblock_encode_p8x8( x264_t *h, int i8 )
         else
             nnz8x8 = array_non_zero( dct8x8 );
 
+        for( i4 = 0; i4 < 4; i4++ )
+            h->mb.cache.non_zero_count[x264_scan8[i8*4+i4]] = nnz8x8;
+
         if( nnz8x8 )
         {
             h->quantf.dequant_8x8( dct8x8, h->dequant8_mf[CQM_8PY], i_qp );
@@ -788,7 +792,6 @@ void x264_macroblock_encode_p8x8( x264_t *h, int i8 )
     }
     else
     {
-        int i4;
         DECLARE_ALIGNED( int16_t, dct4x4[4][4][4], 16 );
         h->dctf.sub8x8_dct( dct4x4, p_fenc, p_fdec );
         h->quantf.quant_4x4( dct4x4[0], h->quant4_mf[CQM_4PY][i_qp], h->quant4_bias[CQM_4PY][i_qp] );
@@ -804,9 +807,23 @@ void x264_macroblock_encode_p8x8( x264_t *h, int i8 )
             for( i4 = 0; i4 < 4 && i_decimate_8x8 < 4; i4++ )
                 i_decimate_8x8 += x264_mb_decimate_score( h->dct.block[i8*4+i4].luma4x4, 16 );
             nnz8x8 = 4 <= i_decimate_8x8;
+
+            for( i4 = 0; i4 < 4; i4++ )
+            {
+                nnz4x4 = ( nnz8x8 ) ? array_non_zero( dct4x4[i4] ) : 0;
+                h->mb.cache.non_zero_count[x264_scan8[i8*4+i4]] = nnz4x4;
+            }
         }
         else
-            nnz8x8 = array_non_zero( dct4x4 );
+        {
+            nnz8x8 = 0;
+            for( i4 = 0; i4 < 4; i4++ )
+            {
+                nnz4x4 = array_non_zero( dct4x4[i4] );
+                h->mb.cache.non_zero_count[x264_scan8[i8*4+i4]] = nnz4x4;
+                nnz8x8 += nnz4x4;
+            }
+        }
 
         if( nnz8x8 )
         {
@@ -827,7 +844,9 @@ void x264_macroblock_encode_p8x8( x264_t *h, int i8 )
         h->dctf.sub4x4_dct( dct4x4, p_fenc, p_fdec );
         h->quantf.quant_4x4( dct4x4, h->quant4_mf[CQM_4PC][i_qp], h->quant4_bias[CQM_4PC][i_qp] );
         h->zigzagf.scan_4x4ac( h->dct.block[16+i8+ch*4].residual_ac, dct4x4 );
-        if( array_non_zero( dct4x4 ) )
+        nnz4x4 = array_non_zero_count( h->dct.block[16+i8+ch*4].residual_ac, 15 );
+        h->mb.cache.non_zero_count[x264_scan8[16+i8+ch*4]] = nnz4x4;
+        if( nnz4x4 )
         {
             h->quantf.dequant_4x4( dct4x4, h->dequant4_mf[CQM_4PC], i_qp );
             h->dctf.add4x4_idct( p_fdec, dct4x4 );